function CustomizationsFloatingCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {
	
	var boundCustomizationUpdateEvent = eventAggregator.subscribe("OnUpdateCustomizationsRepositories", fillFloatingCardRepositories);

	$scope.$on("destroy", function(){
		eventAggregator.unSubscribe("OnUpdateCustomizationsRepositories", boundCustomizationUpdateEvent);
	});
	
	//------Dependencies
	var zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
	var zhFilterPreferences = ApplicationContext.zhFilterPreferences;
	var zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;
	var zhCustomizationsService = ApplicationContext.zhCustomizationsService;
	
	//------ Control viariables
	var editingWidget = $scope.$parent.widget;
	var customizationIndex = 0;
	var floatingControl = editingWidget.floatingControl || {};
	var customizationsWidgetOptions = floatingControl.customizationAction;
	$scope.settingsCardLabel = ScreenService.i18n("Configurations");

	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			} else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}				
			}
		}
	}

	$scope.togglePreferencesCard = function(e){
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};
	editingWidget.togglePreferencesCard = $scope.togglePreferencesCard;

	var preferencesConfigsClick = function(){
		zhLayoutPreferences.startConfig(editingWidget);
	};

	var filtersConfigsClick = function(){
		zhFilterPreferences.startFilterConfig(editingWidget);
	};

	var viewsConfigsClick = function(){
		zhCustomizationPreferences.startViewSettings(editingWidget);
	};

	var preferencesRestoreClick = function(){
		zhLayoutPreferences.restoreDefault(editingWidget);
	};

	var filtersRestoreClick = function(){
		zhFilterPreferences.restoreDefault(editingWidget);
	};

	var viewsRestoreClick = function(){
		zhCustomizationPreferences.restoreDefault(editingWidget);
	};

	function applyFilter(customization){
		zhCustomizationsService.applyFilter(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyLayout(customization){
		zhCustomizationsService.applyLayout(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyVisualization(customization){
		zhCustomizationsService.applyVisualization(customization, editingWidget);
	}

	$scope.isCustomizationApplied = function(customization){
		if(editingWidget.customizationsApplied){
			var appliedCustomization = editingWidget.customizationsApplied[customization.customizationType];
			if(appliedCustomization == customization._id ){
				return true;
			}
		}
	};

	$scope.customizationsTypes = [
		{
			"label": ScreenService.i18n("Views"),
			"type": "view",
			"configsClick": viewsConfigsClick,
			"restoreClick": viewsRestoreClick,
			"applyOnClick": applyVisualization,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Layouts"),
			"type": "layout",
			"configsClick": preferencesConfigsClick,
			"restoreClick": preferencesRestoreClick,
			"applyOnClick": applyLayout,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Filters"),
			"type": "filter",
			"configsClick": filtersConfigsClick,
			"restoreClick": filtersRestoreClick,
			"applyOnClick": applyFilter,
			"dataSource": {
				"data": []
			}
		}
	];

	$scope.setActiveCustomization = function(type) {
		$scope.activeCustomization = $scope.customizationsTypes.filter(function(customizationType) {
			return customizationType.type == type;
		}).shift();
	};

	function fillFloatingCardRepositories(){
		return ZHPromise.all($scope.customizationsTypes.filter(function(customizationType) {
			return Util.isUndefined(editingWidget.floatingControl) || Util.isUndefined(editingWidget.floatingControl.customizationAction) || editingWidget.floatingControl.customizationAction[customizationType.type] !== false;
		}).map(function(customizationType){
			return zhCustomizationsService.getWidgetCustomizations(editingWidget, customizationType.type).then(function(custo){
				insertDefaultOption(custo, customizationType.type);
				customizationType.dataSource.data = custo;
				ScreenService.updateTemplate();
			});
		}));
	}
	fillFloatingCardRepositories();

	function insertDefaultOption(customizations, customizationType){
		var defaultOption = {
			"_id": "default",
			"label": ScreenService.i18n("Original"),
			"customizationType": customizationType,
			"isDefaultOption": true
		};
		if(defaultOption.customizationType == "filter" && zhFilterPreferences){
			defaultOption.filters = zhFilterPreferences.getDefaultFilterProperties(editingWidget) || [];
		}
		customizations.push(defaultOption);
	}

	$scope.previousCustomizationType = function(){
		if(customizationIndex === 0){
			customizationIndex = $scope.customizationsTypes.length - 1;
		} else {
			customizationIndex--;
		}
		updateCustomizationType(customizationIndex);
	};

	$scope.forwardCustomizationType = function(){
		if(customizationIndex === $scope.customizationsTypes.length - 1){
			customizationIndex = 0;
		} else {
			customizationIndex++;
		}
		updateCustomizationType(customizationIndex);
	};
}